﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Runtime.CompilerServices;
using System.Threading;
using MediaPortal.Configuration;
using MediaPortal.Database;
using MediaPortal.GUI.Library;
using MediaPortal.Player;
using SQLite.NET;

namespace MyKindGirls.Database
{
    public class KindGirlsDatabaseSqlLite : IKindGirlsDatabase, IDisposable
    {
        private bool disposed = false;
        private SQLiteClient m_db = null;

        public KindGirlsDatabaseSqlLite()
        {
            Open();
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        private void Open()
        {
            try
            {
                // Maybe called by an exception
                if (m_db != null)
                {
                    try
                    {
                        m_db.Close();
                        m_db.Dispose();
                        Log.Warn("KindGirlsDatabaseSqlLite: Disposing current instance..");
                    }
                    catch (Exception) { }
                }

                // Open database
                try
                {
                    Directory.CreateDirectory(Config.GetFolder(Config.Dir.Database));
                }
                catch (Exception) { }
                m_db = new SQLiteClient(Config.GetFile(Config.Dir.Database, "KindGirlsDatabase.db3"));
                // Retry 10 times on busy (DB in use or system resources exhausted)
                m_db.BusyRetries = 10;
                // Wait 100 ms between each try (default 10)
                m_db.BusyRetryDelay = 100;

                DatabaseUtility.SetPragmas(m_db);
                CreateTables();
            }
            catch (Exception ex)
            {
                Log.Error("KindGirlsDatabaseSqlLite: exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
                Open();
            }
            Log.Info("picture database opened");
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        private bool CreateTables()
        {
            if (m_db == null)
            {
                return false;
            }
            DatabaseUtility.AddTable(m_db, "picture",
                                     "CREATE TABLE picture ( idPicture integer primary key, strLink text, strFilename text, strModelTitle text, strPictureTitle)");
            return true;
        }

        [MethodImpl(MethodImplOptions.Synchronized)]
        public int AddPicture(string strLink, string strFilename, string strModelTitle, string strPictureTitle)
        {
            if (String.IsNullOrEmpty(strFilename))
            {
                Log.Debug("KindGirlsDatabaseSqlLite: filename was null");
                return -1;
            }

            if (m_db == null)
            {
                Log.Debug("KindGirlsDatabaseSqlLite: db is null");
                return -1;
            }
            try
            {
                int lPicId = -1;
                string strLi = strLink;
                string strFile = strFilename;
                string strModelT = strModelTitle;
                string strPictureT = strPictureTitle;
                DatabaseUtility.RemoveInvalidChars(ref strLink);
                DatabaseUtility.RemoveInvalidChars(ref strFile);
                DatabaseUtility.RemoveInvalidChars(ref strModelT);
                DatabaseUtility.RemoveInvalidChars(ref strPictureT);
                string strSQL = String.Format("select * from picture where strFilename = '{0}'", strFile);
                SQLiteResultSet results = m_db.Execute(strSQL);
                if (results != null && results.Rows.Count > 0)
                {
                    lPicId = Int32.Parse(DatabaseUtility.Get(results, 0, "idPicture"));
                    return lPicId;
                }

                // Transactions are a special case for SQLite - they speed things up quite a bit
                strSQL = "begin";
                results = m_db.Execute(strSQL);
                strSQL =
                  String.Format(
                    "insert into picture (idPicture, strLink, strFilename, strModelTitle, strPictureTitle) values(null, '{0}','{1}','{2}','{3}')", strLi, strFile,
                    strModelT, strPictureT);

                results = m_db.Execute(strSQL);
                if (results.Rows.Count > 0)
                {
                    Log.Debug("KindGirlsDatabaseSqlLite: Added to database - {0}", strFile);
                }
                strSQL = "commit";
                results = m_db.Execute(strSQL);

                lPicId = m_db.LastInsertID();

                if (g_Player.Playing)
                {
                    Thread.Sleep(50);
                }
                else
                {
                    Thread.Sleep(1);
                }

                return lPicId;
            }
            catch (Exception ex)
            {
                Log.Error("KindGirlsDatabaseSqlLite: exception err:{0} stack:{1}", ex.Message, ex.StackTrace);
                Open();
            }
            return -1;
        }

        public int ListModels(ref List<string> Models)
        {
            int Count = 0;
            lock (typeof(KindGirlsDatabase))
            {
                if (m_db == null)
                {
                    return 0;
                }
                string strSQL = "select distinct strModelTitle from picture order by 1";
                Log.Debug("KindGirlsDatabaseSqlLite: select distinct strModelTitle from picture order by 1");
                SQLiteResultSet result;
                try
                {
                    result = m_db.Execute(strSQL);
                    if (result != null)
                    {
                        Log.Debug("KindGirlsDatabaseSqlLite: select count: " + result.Rows.Count);
                        for (Count = 0; Count < result.Rows.Count; Count++)
                        {
                            Models.Add(DatabaseUtility.Get(result, Count, 0));
                            Log.Debug("KindGirlsDatabaseSqlLite: model: " + DatabaseUtility.Get(result, Count, 0));
                        }
                    }
                }
                catch (Exception ex)
                {
                    Log.Error("KindGirlsDatabaseSqlLite: exception getting Models err:{0} stack:{1}", ex.Message, ex.StackTrace);
                    Open();
                }
                return Count;
            }
        }

        public int ListFilesForModel(string Model, ref List<string[]> Files)
        {
            int Count = 0;
            lock (typeof(KindGirlsDatabase))
            {
                if (m_db == null)
                {
                    return 0;
                }
                string strSQL = "select strFilename, strPictureTitle from picture where strModelTitle = '" + Model +
                                "' order by 2";
                SQLiteResultSet result;
                try
                {
                    result = m_db.Execute(strSQL);
                    if (result != null)
                    {
                        for (Count = 0; Count < result.Rows.Count; Count++)
                        {
                            Files.Add(new string[2] {DatabaseUtility.Get(result, Count, 0), DatabaseUtility.Get(result, Count, 1)});
                        }
                    }
                }
                catch (Exception ex)
                {
                    Log.Error("KindGirlsDatabaseSqlLite: exception getting Files err:{0} stack:{1}", ex.Message, ex.StackTrace);
                    Open();
                }
                return Count;
            }
        }

        public string GetFilenameForModel(string Model)
        {
            lock (typeof(KindGirlsDatabase))
            {
                if (m_db == null)
                {
                    return "";
                }
                string strSQL = "select distinct strFilename from picture where strModelTitle = '" + Model +
                                "' order by 1";
                SQLiteResultSet result;
                try
                {
                    result = m_db.Execute(strSQL);
                    if (result != null)
                    {
                        Log.Debug("KindGirlsDatabaseSqlLite: select count: " + result.Rows.Count);
                        return DatabaseUtility.Get(result, 0, 0);
                    }
                }
                catch (Exception ex)
                {
                    Log.Error("KindGirlsDatabaseSqlLite: exception getting Files err:{0} stack:{1}", ex.Message, ex.StackTrace);
                    Open();
                }
                return "";
            }
        }

        public int ListFilesForSlideshow(ref List<string> Files)
        {
            int Count = 0;
            lock (typeof(KindGirlsDatabase))
            {
                if (m_db == null)
                {
                    return 0;
                }
                string strSQL = "select distinct strFilename from picture order by 1";
                SQLiteResultSet result;
                try
                {
                    result = m_db.Execute(strSQL);
                    if (result != null)
                    {
                        for (Count = 0; Count < result.Rows.Count; Count++)
                        {
                            Files.Add(DatabaseUtility.Get(result, Count, 0));
                        }
                    }
                }
                catch (Exception ex)
                {
                    Log.Error("KindGirlsDatabaseSqlLite: exception getting Files err:{0} stack:{1}", ex.Message, ex.StackTrace);
                    Open();
                }
                return Count;
            }
        }
        public string DatabaseName
        {
            get
            {
                if (m_db != null)
                {
                    return m_db.DatabaseName;
                }
                return "";
            }
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (!disposed)
            {
                disposed = true;
                if (m_db != null)
                {
                    try
                    {
                        m_db.Close();
                        m_db.Dispose();
                    }
                    catch (Exception) { }
                    m_db = null;
                }
            }
        }

        #endregion
    }
}
