﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Drawing.Imaging;
using System.Globalization;
using System.IO;
using System.Runtime.CompilerServices;
using System.Threading;
using System.Windows.Forms;
using System.Xml.Serialization;
using MediaPortal.Configuration;
using MediaPortal.Database;
using MediaPortal.Dialogs;
using MediaPortal.GUI.Library;
using MediaPortal.Picture.Database;
using MediaPortal.Services;
using MediaPortal.Threading;
using MediaPortal.Util;
using MediaPortal.Player;
using Action = MediaPortal.GUI.Library.Action;
using Layout = MediaPortal.GUI.Library.GUIFacadeControl.Layout;
using WindowPlugins;

using MyKindGirls.Database;
using MediaPortal.GUI.Pictures;


namespace MyKindGirls
{
    [PluginIcons("MyKindGirls.kind.png", "MyKindGirls.kindDisabled.png")]
    public class GuiMyKindGirls : GUIWindow, ISetupForm, IShowPlugin
    {
        #region skin
        [SkinControl(2)]
        protected GUIButtonControl btnViewAs = null;

        [SkinControl(6)]
        protected GUIButtonControl btnSlideShow = null;

        [SkinControl(50)]
        protected GUIFacadeControl facadeView = null;
        #endregion

        #region var declaration
        private string thumbspath = Config.GetFolder(Config.Dir.Thumbs) + "\\KindGirls\\";
        private string cachepath = Config.GetFolder(Config.Dir.Cache) + "\\KindGirls\\";

        private int selectedItemIndex = -1;
        View currentView = View.LargeIcons;

        private List<string> models;
        private List<string> files;

        private IKindGirlsDatabase _database;

        #endregion

        enum View : int
        {
            List = 0,
            Icons = 1,
            LargeIcons = 2,
        }

        public GuiMyKindGirls()
        {
            GetID = (int)74883884;
        }

        public override bool Init()
        {
            bool bResult = Load(GUIGraphicsContext.Skin + @"\mykindgirls.xml");
            _database = MyKindGirls.Database.DatabaseFactory.GetKindGirlsDatabase();
            return bResult;
        }

        public override void DeInit()
        {
            _database.Dispose();
            base.DeInit();
        }

        protected override void OnPageLoad()
        {
            models = new List<string>();

            _database.ListModels(ref models);

            Log.Debug("Models count: " + models.Count);
            foreach (string model in models)
            {
                GUIListItem item = new GUIListItem();
                item.Label = model;
                string fname = _database.GetFilenameForModel(model);
                Log.Debug("item thumbnail: " + cachepath + fname);
                item.ThumbnailImage = cachepath + fname;
                facadeView.Add(item);
            }

            SwitchView();

            base.OnPageLoad();
        }

        protected override void OnPageDestroy(int new_windowId)
        {
            base.OnPageDestroy(new_windowId);
        }

        protected override void OnClicked(int controlId, GUIControl control, MediaPortal.GUI.Library.Action.ActionType actionType)
        {
            #region ViewAs
            if (control == btnViewAs)
            {
                bool shouldContinue = false;
                do
                {
                    shouldContinue = false;
                    switch (currentView)
                    {
                        case View.List:
                            currentView = View.Icons;
                            if (facadeView.ThumbnailLayout == null)
                                shouldContinue = true;
                            else
                                facadeView.CurrentLayout = GUIFacadeControl.Layout.SmallIcons;
                            break;

                        case View.Icons:
                            currentView = View.LargeIcons;
                            if (facadeView.ThumbnailLayout == null)
                                shouldContinue = true;
                            else
                                facadeView.CurrentLayout = GUIFacadeControl.Layout.LargeIcons;
                            break;

                        case View.LargeIcons:
                            currentView = View.List;
                            if (facadeView.ListLayout == null)
                                shouldContinue = true;
                            else
                                facadeView.CurrentLayout = GUIFacadeControl.Layout.List;
                            break;
                    }
                } while (shouldContinue);

                SelectCurrentItem();
                GUIControl.FocusControl(GetID, controlId);
                return;
            }
            #endregion

            if (control == btnSlideShow) // Slide Show
            {
                OnSlideShow(null);
            }

            if (control == facadeView)
            {
                GUIMessage msg = new GUIMessage(GUIMessage.MessageType.GUI_MSG_ITEM_SELECTED, GetID, 0, facadeView.GetID, 0, 0,
                                                null);
                OnMessage(msg);
                int itemIndex = (int)msg.Param1;

                if (actionType == MediaPortal.GUI.Library.Action.ActionType.ACTION_SELECT_ITEM)
                {
                    OnClick(itemIndex);
                }
            }
        }

        private void OnSlideShow(string model)
        {
            GUISlideShow SlideShow = (GUISlideShow)GUIWindowManager.GetWindow((int)Window.WINDOW_SLIDESHOW);
            if (SlideShow == null)
            {
                return;
            }

            if (model == null)
            {
                if (files == null)
                {
                    files = new List<string>();
                    _database.ListFilesForSlideshow(ref files);
                }

                foreach (string file in files)
                {
                    SlideShow.Add(thumbspath + file);
                }
            }
            else
            {
                List<string[]> pictures = new List<string[]>();
                _database.ListFilesForModel(model, ref pictures);
                foreach (string[] picture in pictures)
                {
                    SlideShow.Add(thumbspath + picture[0]);
                }

            }

            SlideShow.StartSlideShow();
            GUIWindowManager.ActivateWindow((int)Window.WINDOW_SLIDESHOW);
        }

        private void OnClick(int itemIndex)
        {
            GUIListItem item = GetSelectedItem();
            if (item == null)
            {
                return;
            }

            selectedItemIndex = GetSelectedItemNo();
            OnSlideShow(item.Label);
        }

        private void OnShowPicture(string strFile)
        {
            GUISlideShow SlideShow = (GUISlideShow)GUIWindowManager.GetWindow((int)Window.WINDOW_SLIDESHOW);
            if (SlideShow == null)
            {
                return;
            }

            SlideShow.Reset();

            /*foreach (string[] url in list)
            {
                SlideShow.Add(picspath + url[2] + "\\" + url[1] + ".jpg");
            }*/

            if (SlideShow.Count > 0)
            {
                GUIWindowManager.ActivateWindow((int)Window.WINDOW_SLIDESHOW);
                SlideShow.Select(strFile);
            }
        }

        private int GetSelectedItemNo()
        {
            return facadeView.SelectedListItemIndex;
        }

        private GUIListItem GetSelectedItem()
        {
            return facadeView.SelectedListItem;
        }

        void SelectCurrentItem()
        {
            int iItem = facadeView.SelectedListItemIndex;
            if (iItem > -1)
            {
                GUIControl.SelectItemControl(GetID, facadeView.GetID, iItem);
            }
            UpdateButtonStates();
        }

        void UpdateButtonStates()
        {
            facadeView.IsVisible = false;
            facadeView.IsVisible = true;
            GUIControl.FocusControl(GetID, facadeView.GetID);

            string strLine = string.Empty;
            View view = currentView;
            switch (view)
            {
                case View.List:
                    strLine = GUILocalizeStrings.Get(101);
                    break;
                case View.Icons:
                    strLine = GUILocalizeStrings.Get(100);
                    break;
                case View.LargeIcons:
                    strLine = GUILocalizeStrings.Get(417);
                    break;
            }
            if (btnViewAs != null)
            {
                btnViewAs.Label = strLine;
            }
        }

        void SwitchView()
        {
            switch (currentView)
            {
                case View.List:
                    facadeView.CurrentLayout = GUIFacadeControl.Layout.List;
                    break;
                case View.Icons:
                    facadeView.CurrentLayout = GUIFacadeControl.Layout.SmallIcons;
                    break;
                case View.LargeIcons:
                    facadeView.CurrentLayout = GUIFacadeControl.Layout.LargeIcons;
                    break;
            }

            UpdateButtonStates(); // Ensure "View: xxxx" button label is updated to suit
        }

        #region ISetupForm Members

        public bool CanEnable()
        {
            return true;
        }

        public string PluginName()
        {
            return "My KindGirls";
        }

        public bool HasSetup()
        {
            return true;
        }
        public bool DefaultEnabled()
        {
            return true;
        }

        public int GetWindowId()
        {
            return GetID;
        }

        public bool GetHome(out string strButtonText, out string strButtonImage,
          out string strButtonImageFocus, out string strPictureImage)
        {
            using (MediaPortal.Profile.Settings xmlreader = new MediaPortal.Profile.Settings("MediaPortal.xml"))

                if (xmlreader.GetValue("general", "myprefix") == "no")
                {
                    strButtonText = "KindGirls";
                }
                else
                {
                    strButtonText = "My KindGirls";
                }
            strButtonImage = String.Empty;
            strButtonImageFocus = String.Empty;
            strPictureImage = "hover_kindgirls.png";
            return true;
        }

        public string Author()
        {
            return "ADULTS ONLY!";
        }

        public string Description()
        {
            return "View the picture archive from kindgirls.com";
        }

        public void ShowPlugin()
        {
            Setup frm = new Setup();
            frm.ShowDialog();
        }

        #endregion

        #region IShowPlugin Members

        public bool ShowDefaultHome()
        {
            return false;
        }

        #endregion
    }
}
